%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                   %
% Implementation of G1m's specification in LL using SELLF           %
%                                                                   %
% Author: Giselle Machado N. Reis                                   %
%                                                            2010   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

module g1m.
accumulate sellf.

g1m L R :- atomize_left L L1, 
           atomize_right R R1,
           append L1 R1 F,
           sellf ((k_elm r R1)::(k_elm l L1):: 
                    (k_elm rules (
                        (sigma A\ sigma B\ ((my_not (bd (A imp B))) , ((bang l (? r (bu A))) , (? l (bd B)))))::
                        (sigma A\ sigma B\ ((my_not (bu (A imp B))) , ((? l (bd A)) @ (? r (bu B)))))::
                        (sigma A\ sigma B\ ((my_not (bd (A and B))) , ((? l (bd A)) ; (? l (bd B)))))::
                        (sigma A\ sigma B\ ((my_not (bu (A and B))) , ((? r (bu A)) , (? r (bu B)))))::
                        (sigma A\ sigma B\ ((my_not (bd (A or B))) , ((? l (bd A)) & (? l (bd B)))))::
                        (sigma A\ sigma B\ ((my_not (bu (A or B))) , ((? r (bu A)) ; (? r (bu B)))))::
                        (sigma A\ ((my_not (bd (forall A))) , (? l (bd (A X)))))::
                        (sigma A\ ((my_not (bu (forall A))) , (pi x\ (? r (bu (A x))))))::
                        (sigma A\ ((my_not (bd (exists A))) , (pi x\ (? l (bd (A x))))))::
                        (sigma A\ ((my_not (bu (exists A))) , (? r (bu (A X)))))::
                        (sigma B\ ((my_not (bd B)) , ((? l (bd B)) @ (? l (bd B)))))::
                        (sigma B\ ((my_not (bd B)) , (bottom)))::
                        (sigma A\ ((my_not (bd A)) , (my_not (bu A))))::
                        (sigma A\ ((bang l (? r (bu A))) , (? l (bd A))))::
                        nil
                   ))::nil)
                   nil
                   F.


atomize_left nil nil.
atomize_left (H::L) ((bd H)::T) :- atomize_left L T.

atomize_right nil nil.
atomize_right (H::L) ((bu H)::T) :- atomize_right L T.

%% Polarity

neg (bd X).
neg (bu X).

%% Subexponentials
%% l and r are unrelated

pred l rules.
pred r rules.

cont rules.
weak rules.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                     %%
%%     Specification of the rules      %%
%%                                     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ==> Left implication
%((my_not (bd (A imp B))) , ((bang l (? r (bu A))) , (? l (bd B))))

% ==> Right implication
%((my_not (bu (A imp B))) , ((? l (bd A)) @ (? r (bu B))))
    
% ==> Left conjunction
%((my_not (bd (A and B))) , ((? l (bd A)) ; (? l (bd B))))
    
% ==> Right conjunction
%((my_not (bu (A and B))) , ((? r (bu A)) , (? r (bu B))))

% ==> Left disjunction
%((my_not (bd (A or B))) , ((? l (bd A)) & (? l (bd B))))

% ==> Right disjunction
%((my_not (bu (A or B))) , ((? r (bu A)) ; (? r (bu B))))

% ==> Left for all
%((my_not (bd (forall A))) , (? l (bd (A X))))

% ==> Right for all
%((my_not (bu (forall A))) , (pi x\ (? r (bu (A x)))))

% ==> Left exists
%((my_not (bd (exists A))) , (pi x\ (? l (bd (A x)))))

% ==> Right exists
%((my_not (bu (exists A))) , (? r (bu (A X))))

% ==> Contraction
%((my_not (bd B)) , ((? l (bd B)) @ (? l (bd B))))

% ==> Weakening
%((my_not (bd B)) , (bottom))
    
% ==> Initial 1
%((my_not (bd A)) , (my_not (bu A)))

% ==> Initial 2
%((bang l (? r (bu A))) , (? l (bd A)))

