%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                   %
% Implementation of LJQ's specification in LL using SELLF           %
%                                                                   %
% Author: Giselle Machado N. Reis                                   %
%                                                            2010   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

module ljq.
accumulate sellf.

ljq L R :-
    atomize_left L L1,
    atomize_right R R1,
    append L1 R1 F,
    sellf ((k_elm l L1)::(k_elm r R1)::(k_elm f nil)::
             (k_elm rules (
                (sigma A\ ((my_not (bd A)) , (my_not (bu A))))::
                (((my_not (bd btm)) , top))::
                (sigma A\ sigma B\ ((my_not (bd (A imp B))) , ((bang l (? f (bu A))) , (bang r (? l (bd B))))))::
                (sigma A\ sigma B\ ((my_not (bu (A imp B))) , (bang l ((? l (bd A)) @ (? r (bu B))))))::
                (sigma A\ sigma B\ ((my_not (bd (A or B))) , ((bang r (? l (bd A))) , (bang r (? l (bd B))))))::
                (sigma A\ sigma B\ ((my_not (bu (A or B))) , (bang r ((? r (bu A)) @ (? r (bu B))))))::
                (sigma A\ sigma B\ ((my_not (bd (A and B))) , (bang r ((? l (bd A)) @ (? l (bd B))))))::
                (sigma A\ sigma B\ ((my_not (bu (A and B))) , ((bang r (? f (bu A))) , (bang r (? f (bu B))))))::
                nil)
             )::nil
            )
            nil
            F.

atomize_left nil nil.
atomize_left (H::L) ((bd H)::T) :- atomize_left L T.

atomize_right nil nil.
atomize_right (H::L) ((bu H)::T) :- atomize_right L T.
    
%% Polarity

neg (bd X).
neg (bu X).

%% Subexponentials

cont r.
weak r.
cont l.
weak l.
cont rules.
weak rules.

pred r l.
pred l rules.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                     %%
%%     Specification of the rules      %%
%%                                     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ==> Initial
%((my_not (bd A)) , (my_not (bu A)))

% ==> Left bottom
%((my_not (bd btm)) , top)

% ==> Left implication
%((my_not (bd (A imp B))) , ((bang l (? f (bu A))) , (bang r (? l (bd B)))))

% ==> Right implication
%((my_not (bu (A imp B))) , (bang l ((? l (bd A)) @ (? r (bu B)))))

% ==> Left disjunction
%((my_not (bd (A or B))) , ((bang r (? l (bd A))) , (bang r (? l (bd B)))))

% ==> Right disjunction
%((my_not (bu (A or B))) , (bang r ((? r (bu A)) @ (? r (bu B)))))

% ==> Left conjunction
%((my_not (bd (A and B))) , (bang r ((? l (bd A)) @ (? l (bd B)))))

% ==> Right conjunction
%((my_not (bu (A and B))) , ((bang r (? f (bu A))) , (bang r (? f (bu B)))))




