%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                   %
% Interpreter for the linear logic with subexponentials (SELLF)     %
% Signature file.                                                   %
%                                                                   %
% Author: Giselle Machado N. Reis                                   %
%                                                            2010   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sig sellf.

%% Top level predicate
type sellf (list k) -> (list o) -> (list o) -> o.

%% Type definitions

kind pair type -> type -> type. %% pair type constructor
kind k type.
type k_elm index -> list E -> k. % This way my context K is polymorphic =) That ought to make things easier
                                                
kind index type.  % Subexponential index type

type at index -> A -> o.

%%%%% The logical connectives for SELLF 
%% Some are commented out because they already exist in Teyjus

%% Asynchronous operatos

%type    &       o -> o -> o.
 type    @       o -> o -> o.
 type    ?       index -> A -> o. % with index
%type    pi      o -> o.
 type    top     o.   % a kind of true
 type    bottom  o.   % a kind of false
 type    create  index -> o -> o.

 infixl  @     8.

%% Synchronous operators

%type    ;       o -> o -> o.
%type    ,       o -> o -> o.
%type    sigma  o -> o.
 type    bang    index -> o -> o. % with index
 type bang_empty index -> o -> o. % with index
%type    true     o.   % a kind of true
 type    zero    o.   % a kind of false (never used in the inference rules... why?) 
 type    inst    index -> o -> o.

%% Function type definitions

% 'list (pair index (list o))': a list of elements of the k function ( pair index (list o) )
% 'list o' (fst): a list of bounded formulas
% 'list o' (snd): the set on the right side of the arrow 
%   (in the asynchronous phase, the first formula is the focused one, 
%   in the synchronous phase, this set should have only one formula)
% 'prove' yelds true or false
type prove_a int -> (list k) -> (list o) -> (list o) -> int -> o.
type prove_s int -> (list k) -> (list o) -> o -> int -> o.

% This is necessary so that I can have other types in K, like integers
type my_not A -> A.

type pos            o -> o.
type neg            o -> o.
type non_literal    o -> o.
type literal        o -> o.
type cont           index -> o.
type weak           index -> o.
type pred           index -> index -> o.

%% Clauses for functions involving K
type k_geq              index -> (list k) -> (list k) -> o.
type k_subset           list index -> (list k) -> (list k) -> o.
type k_subset2          list index -> (list k) -> (list k) -> o.
type k_insert           index -> A -> (list k) -> (list k) -> o.
type k_split            (list k) -> (list k) -> (list k) -> o.
type k_get              index -> (list k) -> list A -> o.
type k_not_weakenable   (list k) -> list A -> o.
type k_insert_end       index -> A -> (list k) -> (list k) -> o.

%% Clauses of side conditions
type condition1         (list k) -> o.
type condition_dl       index -> o.
type not_condition_dl   index -> o.
type condition_and      (list k) -> (list k) -> o.
type condition_bang     index -> (list k) -> o.
type condition_init     (list k) -> list o -> A -> o.
type condition_init_2   (list k) -> list o -> index -> A -> o.

%% Auxiliars
type is_in          A -> list A -> o.
type is_in_k        A -> (list k) -> o.
type is_index       index -> (list k) -> o.
type has_index      A -> index -> (list k) -> o.
type is_empty       index -> (list k) -> o.
type append         list A -> list A -> list A -> o.
type split list     A -> list A -> list A -> o.
type insert_nd      A -> list A -> list A -> o.
type s              int -> int.

%% I/O
kind printcommand    type.
type pc              A -> printcommand.
type nl, fl          printcommand.
type printf          list printcommand -> o.

type print_context  (list k) -> (list o) -> o.
type print_k        (list k) -> o.
type print_k_rec    (list k) -> o.
type print_g        (list o) -> o.



